/**
 * Copyright (C) 2009-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.network;

import java.io.Serializable;

/**
 * <p>
 * Describes a specific firewall rule allowing access through the target firewall.
 * </p>
 * @author George Reese @ enStratus (http://www.enstratus.com)
 */
public class FirewallRule implements Serializable {
    private static final long serialVersionUID = -3830971132733183138L;
    
    static public String getRuleId(String providerFirewallId, String cidr, Direction direction, Protocol protocol, int startPort, int endPort) {
        return providerFirewallId + ":" + cidr + ":" + direction + ":" + protocol + ":" + startPort + ":" + endPort;
    }
    
    private String     cidr;
    private Direction  direction = Direction.INGRESS;
    private int        endPort;
    private String     firewallId;
    private Permission permission = Permission.ALLOW;
    private Protocol   protocol;
    private String     providerRuleId;
    private int        startPort;
    
    /**
     * Constructs an empty firewall rule for marshalling purposes.
     */
    public FirewallRule() { }

    /**
     * Constructs a firewall rule with the specified rule state.
     * @param firewallId the unique ID of the firewall to which this rule belongs
     * @param cidr the CIDR naming the source for this rule
     * @param protocol the protocol supporting this rule
     * @param startPort the start port for the access behind this rule
     * @param endPort the end port for the access behind this rule
     */
    public FirewallRule(String firewallId, String cidr, Protocol protocol, int startPort, int endPort) {
        this.firewallId = firewallId;
        this.cidr = cidr;
        this.protocol = protocol;
        this.startPort = startPort;
        this.endPort = endPort;
    }
    
    /**
     * The source IP address or range of addresses in CIDR format. IP addresses matching
     * this CIDR have positive access to any server protected by this firewall to the port
     * range specified over the protocol specified.
     * @return the source CIDR for this rule
     */
    public String getCidr() {
        return cidr;
    }
    
    /**
     * @return the last port in the range of ports allowed by this rule
     */
    public int getEndPort() {
        return endPort;
    }
    
    /**
     * @return the unique provider ID for the firewall behind this rule
     */
    public String getFirewallId() {
        return firewallId;
    }
    
    /**
     * @return the network protocol to allow through to the target ports
     */
    public Protocol getProtocol() {
        return protocol;
    }
    
    /**
     * @return the first port in the range of ports allowed by this rule
     */
    public int getStartPort() {
        return startPort;
    }
    
    /**
     * Used only for marshalling and not to be used programatically.
     * @param cidr the source CIDR for the rule being marshalled
     */
    public void setCidr(String cidr) {
        this.cidr = cidr;
    }
    
    /**
     * Used only for marshalling and not to be used programatically.
     * @param endPort the end port for the rule being marshalled
     */
    public void setEndPort(int endPort) {
        this.endPort = endPort;
    }
    
    /**
     * Used only for marshalling and not to be used programatically.
     * @param firewallId the unique provider ID for the firewall for the rule being marshalled
     */
    public void setFirewallId(String firewallId) {
        this.firewallId = firewallId;
    }
    
    /**
     * Used only for marshalling and not to be used programatically.
     * @param protocol the protocol for the rule being marshalled
     */
    public void setProtocol(Protocol protocol) {
        this.protocol = protocol;
    }
    
    /**
     * Used only for marshalling and not to be used programatically.
     * @param startPort the start port for the rule being marshalled
     */
    public void setStartPort(int startPort) {
        this.startPort = startPort;
    }

    public void setDirection(Direction direction) {
        this.direction = direction;
    }

    public Direction getDirection() {
        return direction;
    }

    public void setPermission(Permission permission) {
        this.permission = permission;
    }

    public Permission getPermission() {
        return permission;
    }

    public void setProviderRuleId(String providerRuleId) {
        this.providerRuleId = providerRuleId;
    }

    public String getProviderRuleId() {
        if( providerRuleId == null ) {
            return FirewallRule.getRuleId(getFirewallId(), getCidr(), getDirection(), getProtocol(), getStartPort(), getEndPort());
        }
        return providerRuleId;
    }
    
    public String toString() {
        return (direction + "/" + permission + ": " + cidr + "->" + protocol + ":" + startPort + "-" + endPort);
    }
}
